import datetime

import pytest


@pytest.fixture(scope="session")
def comic_list_response():
    return {
        "comics": [
            {
                "publisher": "IMAGE COMICS",
                "description": '"LEGACY," Part Five-The third arc of the Eisner Award-winning BITTER ROOT comes to an epic conclusion that will decide the fate of humanity. For the Sangerye family, it means making another sacrifice while searching for hope during hopeless times.',
                "title": "BITTER ROOT #15 CVR A GREENE (MR)",
                "price": "$3.99",
                "creators": "(W) David Walker, Chuck Brown (A/CA) Sanford Greene",
                "release_date": "2021-08-11",
                "diamond_id": "MAY210139",
            },
            {
                "publisher": "IMAGE COMICS",
                "description": '"LEGACY," Part Five-The third arc of the Eisner Award-winning BITTER ROOT comes to an epic conclusion that will decide the fate of humanity. For the Sangerye family, it means making another sacrifice while searching for hope during hopeless times.',
                "title": "BITTER ROOT #15 CVR B CONLEY CURIEL (MR)",
                "price": "$3.99",
                "creators": "(W) David Walker, Chuck Brown (A) Sanford Greene (CA) Chase Conley, Charissa Curiel",
                "release_date": "2021-08-11",
                "diamond_id": "MAY210140",
            },
        ]
    }


@pytest.fixture(scope="session")
def sb_dates_response():
    return {
        "dates": [
            "2021-07-29.",
            "2021-08-04",
            "2021-08-11",
            "2021-08-18",
            "2021-08-25",
        ]
    }


@pytest.fixture(scope="session")
def sb_cleaned_dates():
    return [
        datetime.date(2021, 8, 4),
        datetime.date(2021, 8, 11),
        datetime.date(2021, 8, 18),
        datetime.date(2021, 8, 25),
    ]



from serifan import __version__, api, session


def test_version():
    assert __version__ == "0.1.0"


def test_api():
    sb = None
    try:
        sb = api()
    except Exception as exc:
        print("serifan.api() raised {} unexpectedly!".format(exc))

    assert sb.__class__.__name__ == session.Session.__name__



from marshmallow import ValidationError

from serifan import exceptions, comic


class ComicsList:
    def __init__(self, response):
        self.comics = []

        schema = comic.ComicSchema()
        for issue_dict in response["comics"]:
            try:
                result = schema.load(issue_dict)
            except ValidationError as error:
                raise exceptions.ApiError(error)

            self.comics.append(result)

    def __iter__(self):
        return iter(self.comics)

    def __len__(self):
        return len(self.comics)

    def __getitem__(self, index: int):
        return self.comics[index]



from marshmallow import INCLUDE, Schema, fields, post_load


class Comic:
    def __init__(self, **kwargs):
        for k, v in kwargs.items():
            setattr(self, k, v)


class ComicSchema(Schema):
    """ Schema for Comic api """

    publisher = fields.Str()
    description = fields.Str()
    title = fields.Str()
    price = fields.Str()
    creators = fields.Str()
    release_date = fields.Date(format="%Y-%m-%d")
    diamond_id = fields.Str()

    class Meta:
        unknown = INCLUDE

    @post_load
    def make_object(self, data, **kwargs):
        """
        Make the comic object.
        :param data: Data from Shortboxed response.
        :returns: :class:`Comic` object
        :rtype: Comic
        """
        return Comic(**data)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['serifan']

package_data = \
{'': ['*']}

install_requires = \
['marshmallow>=3.13.0,<4.0.0', 'requests>=2.26.0,<3.0.0']

extras_require = \
{'docs': ['sphinx-rtd-theme>=0.5.2,<0.6.0', 'sphinxcontrib-napoleon>=0.7,<0.8']}

setup_kwargs = {
    'name': 'serifan',
    'version': '0.1.2',
    'description': 'Python wrapper for the Shortboxed API',
    'long_description': '=======\nSerifan\n=======\n\n.. image:: https://img.shields.io/badge/code%20style-black-000000.svg\n    :target: https://github.com/psf/black\n\nQuick Description\n-----------------\nA python wrapper for the shortboxed.com_ API.\n\n.. _shortboxed.com: https://shortboxed.com/\n\nInstallation\n------------\n\nPyPi\n~~~~\n\n.. code:: bash\n\n  $ pip3 install --user serifan\n\nExample Usage\n-------------\n.. code-block:: python\n\n    import serifan\n\n    m = serifan.api()\n\n    # Get this weeks comic releases.\n    results = m.new_releases()\n\n    # Print the results\n    for i in results:\n        print(f"{i.title}")\n\nDocumentation\n-------------\n- `Read the project documentation <https://serifan.readthedocs.io/en/latest/>`_\n \nBugs/Requests\n-------------\n  \nPlease use the `GitHub issue tracker <https://github.com/bpepple/serifan/issues>`_ to submit bugs or request features.\n\nLicense\n-------\n\nThis project is licensed under the `GPLv3 License <LICENSE>`_.\n',
    'author': 'Brian Pepple',
    'author_email': 'bdpepple@gmail.com',
    'maintainer': 'Brian Pepple',
    'maintainer_email': 'bdpepple@gmail.com',
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from datetime import date
from datetime import datetime as dt
from typing import List


def list_strings_to_dates(lst) -> List[date]:
    # One of the string dates in the api has a period at the end.
    return [dt.strptime(d.strip("."), "%Y-%m-%d").date() for d in lst["dates"]]
